function sfg = changeoperand(sfg, operandname, number, operanddata, operandtype)
% This function change an operand in a signal flow graph.
% The properties that can be changed is the data and the type of the
% operand, i.e., input and output nodes can not be changed.
% If only the type is to be changed operanddata can either be an empty
% vector or simply be left out.
% 
% Inputs: sfg - signal flow graph
%         operandname - operand name as a string
%         number - operand ID number
%         operanddata - operand data in a row vector
%         operandtype - operand type as a string
% Output: sfg - updated signal flow graph

if nargin < 4
   error('Not enough input arguments');
end
if nargin > 5
   error('Too many input arguments');
end
if nargin == 4
   if ischar(operanddata)
      operandtype = operanddata;
      operanddata = [];
   else
      operandtype = [];
   end
end

% Load operand mapping
operandmapping;
ops = operandstructure;
op = getidfromname(operandname);
shift_op = getidfromname('shift',ops);
quant_op = getidfromname('quant',ops);
overflow_op = getidfromname('overflow',ops);

if ~ops(op).changeable
   error('The %s operand can not be changed!', operandname);
end

% Find row in sfg
id_op = find(op == sfg(:, 1));
id_no = find(number == sfg(:, 2));
index = 1;
while isempty(find(id_op(index) == id_no))
   index = index + 1;
   if index > length(id_op)
      error('The operand could not be found!');
   end
end
row_op = id_op(index);

% Get format of arguments
d = 0; g = 1; s = 2;
argformat = ops(op).argformat;
id_arg = (3 + sum(ops(op).inout)):(1 + length(argformat));
type_p = length(find(s == argformat));
data_p = length(id_arg) - type_p;

% Operand data
data = [];
if ~isempty(operanddata)
   if ~data_p
      warning('The data can not be changed for the %s operand!', operandname);
   else
      data = operanddata;
      if length(data) ~= data_p
         error('The input argument operanddata contain wrong number of elements!');
      end
   end
elseif data_p
   data = sfg(row_op, id_arg(1:end - type_p));
end

% Operand type
type = [];
if ~isempty(operandtype)
   if ~type_p
      warning('The type can not be changed for the %s operand!', operandname);
   else
      switch op
         case shift_op
            switch lower(operandtype)
               case 'left', type = shift_left;
               case 'right', type = shift_right;
               otherwise, error('Unknown shift type');
            end
         case quant_op
            switch lower(operandtype)
               case {'truncation', 'trunc'}, type = quant_truncation;
               case {'rounding', 'round'}, type = quant_rounding;
               case {'magnitudetruncation', 'magtrunc'}, type = quant_magnitudetruncation;
               otherwise, error('Unknown quantization type');
            end
         case overflow_op
            switch lower(operandtype)
               case 'twosc', type = overflow_twosc;
               case {'saturation', 'sat'}, type = overflow_saturation;
               otherwise, error('Unknown overflow type');
            end
         otherwise, error('Unknown operand');
      end
   end
elseif type_p
   type = sfg(row_op, id_arg(end));
end

if ops(op).isadaptor
    switch lower(operandtype)
        case {'series', 'ser'}, type = adaptor_ser;
        case {'parallel', 'par'}, type = adaptor_par;
        case {'symmetric', 'sym'}, type = adaptor_sym;
        otherwise, error('Unknown adaptor type');
    end
end
% Update operand
sfg(row_op, id_arg) = [data, type];

