function [filename] = dotsfgplot(sfg, format, style, filename)
%DOTSFGPLOT Plot an SFG using the Graphwiz program dot
%
% Inputs: sfg - signal flow graph
%         [format] - 'dot' to generate dot-file, 'eps' (default) to generate eps file
%         [style] - 'dot' to use dot layout, 'neato' to use neato layout,
%         'fdp' to use fdp layout
%         [filename] - file to store the result in
%
% Output: filename - name of file that is generated

operandmapping;
s = operandstructure;
if nargin <= 3
    % Create file name
    filename = sprintf('dotsfgplot_%s', datestr(datetime('now'),'yy_mm_dd_HH_MM_SS_FFF'));
end
% Set default values
if nargin <= 2
    style = 'dot';
end
if nargin == 1
    format = 'eps';
end

% Open file
fid = fopen(sprintf('%s.dot',filename),'w');
% Starting line
fprintf(fid, 'digraph SFG {\n');
% Left to right is probably the better way to draw and SFG
fprintf(fid, 'rankdir = "LR";\n');
% Generate operation nodes
for op = 1:size(sfg,1)
    fprintf(fid, '%s_%d [shape=%s, label="%s"];\n', s(sfg(op,1)).name, sfg(op,2), s(sfg(op,1)).dotshape, s(sfg(op,1)).dotname(sfg(op,:)));
end
% Generate edges
for node = getnodelist(sfg)'
    [source,destinationnodes]=getsourceanddesitationfromnode(sfg,node,s);
    % Node connected to at least two inputs?
    if size(destinationnodes,1) >= 2
        % Intermediate node
        fprintf(fid, 'node_%d [label="", shape=circle, style=filled, color=black, height=0.1];\n', node);
        sourceio = s(sfg(source(1,1),1)).inout;
        % Connect source to intermediate node
        if sourceio(1,2) <= 1
            fprintf(fid, '%s_%d -> node_%d [label="%d" dir="none"];\n', s(sfg(source(1,1),1)).name, sfg(source(1,1),2), node, node);
        else
            fprintf(fid, '%s_%d -> node_%d [label="%d (O: %d)" dir="none"];\n', s(sfg(source(1,1),1)).name, sfg(source(1,1),2), node, node, source(1,2));
        end
        % Connect intermeditate node to inputs
        for destrow = 1:size(destinationnodes,1)
            destio = s(sfg(destinationnodes(destrow,1),1)).inout;
            if destio(1,2) <= 1
                fprintf(fid, 'node_%d -> %s_%d;\n', node, s(sfg(destinationnodes(destrow,1),1)).name, sfg(destinationnodes(destrow,1),2));
            else
                fprintf(fid, 'node_%d -> %s_%d [label="I: %d"];\n', node, s(sfg(destinationnodes(destrow,1),1)).name, sfg(destinationnodes(destrow,1),2), destinationnodes(destrow,2));
            end
        end
    else
        % One source to one destination
        sourceio = s(sfg(source(1,1),1)).inout;
        destio = s(sfg(destinationnodes(1,1),1)).inout;
        if sourceio(1,2) <= 1
            if destio(1,1) <= 1
                fprintf(fid, '%s_%d -> %s_%d [label="%d"];\n', s(sfg(source(1,1),1)).name, sfg(source(1,1),2), s(sfg(destinationnodes(1),1)).name, sfg(destinationnodes(1),2), node);
            else
                fprintf(fid, '%s_%d -> %s_%d [label="%d (I: %d)"];\n', s(sfg(source(1,1),1)).name, sfg(source(1,1),2), s(sfg(destinationnodes(1),1)).name, sfg(destinationnodes(1),2), node, destinationnodes(1,2));
            end
        else
            if destio(1,1) <= 1
                fprintf(fid, '%s_%d -> %s_%d [label="%d (O: %d)"];\n', s(sfg(source(1,1),1)).name, sfg(source(1,1),2), s(sfg(destinationnodes(1),1)).name, sfg(destinationnodes(1),2), node, source(1,2));
            else
                fprintf(fid, '%s_%d -> %s_%d [label="%d (O: %d, I: %d)"];\n', s(sfg(source(1,1),1)).name, sfg(source(1,1),2), s(sfg(destinationnodes(1),1)).name, sfg(destinationnodes(1),2), node, source(1,2), destinationnodes(1,2));
            end
        end
    end
end

fprintf(fid,'}\n');
fclose(fid);
%disp(nargout)
% Make sure we output the correct thing
if strcmp(format,'dot')
    % Return dot filename
    filename = sprintf('%s.dot',filename);
else
    if strcmp(style, 'dot')
        system(sprintf('dot -Teps %s.dot > %s.eps', filename, filename));
    elseif strcmp(style, 'neato')
        system(sprintf('neato -Goverlap=false -Teps %s.dot > %s.eps', filename, filename));
    else
        system(sprintf('fdp -Goverlap=false -Teps %s.dot > %s.eps', filename, filename));
    end
    if not(nargout)
        system(sprintf('gv %s.eps &', filename, filename));
    end
    % Remove the temporary .dot-file
    prevState = recycle('off'); % turn recycle off to permanently delete files
    delete(sprintf('%s.dot',filename))
    recycle(prevState);
    % Return eps filename
    filename=sprintf('%s.eps',filename);
end
end
