function [newsfg, newnode] = insertoperand(sfg, op_str, number, node, op_data, type_str, dest_op_str, dest_op_number, dest_op_input_number)
% This function adds an operand to a signal flow graph
%
% Inputs: sfg - signal flow graph
%         op_str - operand string
%         number - operand number for inserted operand
%         node - node to insert at
%         op_data - operand arguments
%         type_str - operand type
%         [dest_op_str] - destination operand string
%         [dest_op_number] - destination operand number
%         [[dest_op_input_number]] - destination operand input number
% Output: newsfg - signal flow graph
%         newnode - number of the inserted node

if nargin < 4
    error('Not enough input arguments');
end
if nargin > 9
    error('Too many input arguments');
end
if nargin < 5
    op_data = [];
end

% Load operand mapping
operandmapping;
s = operandstructure;
op = getidfromname(op_str);
if ~all(s(op).inout == [1 1])
    error('Can only insert operands with one input and one output');
end

shift_op = getidfromname('shift',s);
quant_op = getidfromname('quant',s);
overflow_op = getidfromname('overflow',s);


type = [];

switch op
    case shift_op
        switch lower(type_str)
            case 'left', type = shift_left;
            case 'right', type = shift_right;
            otherwise, error('Unknown shift type');
        end
    case quant_op
        switch lower(type_str)
            case 'truncation', type = quant_truncation;
            case 'rounding', type = quant_rounding;
            case 'magnitudetruncation', type = quant_magnitudetruncation;
            otherwise, error('Unknown quantization type');
        end
    case overflow_op
        switch lower(type_str)
            case 'twosc', type = overflow_twosc;
            case 'saturation', type = overflow_saturation;
            otherwise, error('Unknown overflow type');
        end
end

newnode = getfreenode(sfg);

op_arg = [number, node, newnode, op_data, type];

if nargin <= 6
    % Find operations using node and change to newnode
    for row = 1:size(sfg,1)
        noinout = s(sfg(row,1)).inout;
        for inputcnt = 1:noinout(1)
            input = 2 + inputcnt;
            if sfg(row, input) == node
                sfg(row, input) = newnode;
            end
        end
    end
else
    % Only insert at the input of a specific operation
    dest_op = getidfromname(dest_op_str);
    for row = 1:size(sfg,1)
        if sfg(row,1) == dest_op && sfg(row,2) == dest_op_number
            noinout = s(sfg(row,1)).inout;
            if nargin >= 9
                % Only change one of the inputs
                if (noinout(1) >= dest_op_input_number)
                    input = 2 + dest_op_input_number;
                    if sfg(row, input) == node
                        sfg(row, input) = newnode;
                    else
                        error('That input is not connected to the original node')
                    end
                else
                    error('Operand does not have that many inputs')
                end
            else
                for inputcnt = 1:noinout(1)
                    input = 2 + inputcnt;
                    if sfg(row, input) == node
                        sfg(row, input) = newnode;
                    end                    
                end
            end
        end
    end
end

newsfg = addrow(sfg, [op, op_arg]);
