function [mvlist, instantvariables] = memorypartitioning(mv, readports, writeports, concurrentports)
%mvlist = memorypartitioning(mv, readports, writeports, concurrentports)
%
%mvlist = {mv1, mv2, ...}
%mv = [scheduletime; variableinfo1; variableinfo2; ...]
% variableinfo = [
% starttime endtime sourceoperand number port 
% consumetime1 consumeoperand1 number1 port1 
% consumetime2 consumeoperand2 number2 port2
% ... ]
%readports is the number of read ports of the memories
%writeports is the number of write ports of the memories
%concurrentports is the number of concurrent memory accesses, i.e. require
%  min(reads, writes) <= concurrentports for all time slots and each memory

schtime = mv(1,1);

nummems = 1;

% Contains the number of allocated reads and writes for each memory and time
% slot. The matrices contain one row for each memory.
readslots = zeros(1, schtime);
writeslots = zeros(1, schtime);


variables=mv(2:end,:);
[~,idx]=sort(sum(isnan(variables')));
allvariables=variables(idx,:);



% If we have a second output argument, divide variables into ordinary
% variables and zero-time variables
if(nargout > 1)
    
    variables = [];
    instantvariables = mv(1,:);
    for var = 1:size(allvariables, 1)

        varinfo = allvariables(var, :);

        starttime = varinfo(1);
        endtime = varinfo(2);

        % Unsure if this check covers all cases, seems OK so far...
        if(starttime == endtime)
            instantvariables = [instantvariables; varinfo];
        else
            variables = [variables; varinfo];
        end

    end
else
   variables = allvariables;
end
    
    
numvars = size(variables, 1);
varlist = 1:numvars;
memvars = {};

while ~isempty(varlist)
    worstmems = nummems + 1;
    
    for var = varlist
        % Extract the variable
        varinfo = variables(var, :);
        
        % Determine the write and read times
        writetime = varinfo(1) + 1;

        % Get all the unique read times for this variable
        readtimes = unique(varinfo(6:4:end) + 1);
        % If the writetime is equal to any of the read times, remove it
        % from the readtimes
        readtimes = readtimes(~isnan(readtimes));
        readtimes = readtimes(readtimes ~= writetime);
        
        
        foundmems = 0;
        foundmem = 0;
        
        % Try to fit the variable in the already allocated memories
        for mem = 1:nummems
            
            % Check if we can allocate the variable to this memory.
            % Otherwise, go to the next memory
            if writeslots(mem, writetime) + 1 > writeports
                continue;
            end
            
            % Same as above for read operations
            if any(readslots(mem, readtimes) + 1 > readports)
                continue;
            end
            
            % Check the number of concurrent accesses
            accesses = readslots(mem, :) + writeslots(mem, :);
            accesses(writetime) = accesses(writetime) + 1;
            accesses(readtimes) = accesses(readtimes) + 1;
            
            % If we do too many concurrent accesses, check the next memory
            if any(accesses > concurrentports)
                continue;
            end
            
            % No violations, mark as possible
            foundmems = foundmems + 1;
            
            if foundmem == 0
                foundmem = mem;
            end
        end
        
        if foundmems < worstmems
            choosevar = var;
            worstmems = foundmems;
            varmem = foundmem;
            % break;
        end
    end
    
    varinfo = variables(choosevar, :);
    
    writetime = varinfo(1) + 1;
    readtimes = unique(varinfo(6:4:end) + 1);
    readtimes = readtimes(~isnan(readtimes));
    readtimes = readtimes(readtimes ~= writetime);
    
    % Allocate choosevar to memory mem or a new memory if worstmems == 0
    if worstmems == 0
        %disp(sprintf('elected %d for new memory', choosevar));
        % Allocate a new memory
        nummems = nummems + 1;
        mem = nummems;
        readslots(mem, :) = zeros(1, schtime);
        writeslots(mem, :) = zeros(1, schtime);
        
        % Update the number of reads and writes
        readslots(mem, readtimes) = readslots(mem, readtimes) + 1;
        writeslots(mem, writetime) = writeslots(mem, writetime) + 1;
        
    else
        %disp(sprintf('elected %d for memory %d, choices: %d', choosevar, varmem, worstmems));
        % Found a fit in one of the allocated memories, update it
        
        mem = varmem;
        
        readslots(mem, readtimes) = readslots(mem, readtimes) + 1;
        writeslots(mem, writetime) = writeslots(mem, writetime) + 1;
    end
    
    % Update the memory variables for the memory
    if length(memvars) < mem
        memvars{mem} = [];
    end
    memvars{mem} = [memvars{mem} choosevar];
    
    % Remove the variable from the list of remaining variables
    varlist(varlist == choosevar) = [];
end

% Create the output cell data
mvlist = cell(1, nummems);

for mem = 1:nummems
    mvpart = zeros(length(memvars{mem}) + 1, size(variables, 2));
    mvpart(2:end,:) = variables(memvars{mem}, :);
    mvpart(1,1) = schtime;
    mvpart(1,2:end) = NaN;
    mvlist{mem} = mvpart;
end

