function [outputs, outputids, registers, registerids, nodes, nodeids] ...
    = simulate(sfg, inputvalues, inputids, initialregisters, initialregisterids, wordlength)
% SIMULATE  Simulate a signal flow graph
%    [OUTPUTS, OUTPUTIDS, REGISTERS, REGISTERIDS, NODES, NODEIDS]
%    = SIMULATE(SFG, INPUTVALUES, INPUTIDS, INITIALREGISTER, REGISTERIDS, WORDLENGTH)
%
%
%    SFG - Signal flow graph
%    INPUTVALUES - Vector or matrix with values to apply to the inputs
%    INPUTNODES - Vector with node numbers for the inputs
%    INITIALREGISTER - Vector with initial values for registers
%    INITIALREGISTERIDS - Ids for registers with initial values
%    WORDLENGTH - Optimal wordlength for finite wordlength simulation

% Copyright 2004-2006
%                Electronics Systems, Dept of EE,
%                Linkoping University, SE-581 83 Linkoping
%                Sweden
%

if (nargin < 2)
    error('Not enough input arguments')
end

% Load operand mapping
s = operandstructure;
in_op = getidfromname('in',s);
out_op = getidfromname('out',s);
add_op = getidfromname('add',s);
sub_op = getidfromname('sub',s);
constmult_op = getidfromname('constmult',s);
delay_op = getidfromname('delay',s);
quant_op = getidfromname('quant',s);
overflow_op = getidfromname('overflow',s);
negate_op = getidfromname('negate',s);
shift_op = getidfromname('shift',s);
twoport_op = getidfromname('twoport',s);
op_threeport = getidfromname('threeport',s);
op_fourport = getidfromname('fourport',s);
butterfly2_op = getidfromname('butterfly2',s);
constdivision_op = getidfromname('constdiv',s);
mux_op = getidfromname('mux',s);
demux_op = getidfromname('demux',s);
constmac_op = getidfromname('constmac',s);
mult_op = getidfromname('mult',s);
division_op = getidfromname('division',s);
reciprocal_op = getidfromname('reciprocal',s);
mac_op = getidfromname('mac',s);
constant_op = getidfromname('constant',s);
addsub_op = getidfromname('addsub',s);
conjugate_op = getidfromname('conjugate',s);
operandmapping;

% Computable order
sfg=sortsfg(sfg);

% Move registers first
regidx=find(sfg(:,1) == delay_op);
regsfg=sfg(regidx,:);
sfg(regidx,:)=[];
sfg=[regsfg;sfg];

% Check inputs
if(nargin < 3)
    inputids = sfg(sfg(:, 1) == in_op, 2);
    numin = length(inputids);
    if ~numin
        error('There is no input in the SFG');
    end
    if numin ~= 1
        error('More than one input available, please specify stimuli-input mapping')
    end
end

% Finite wordlength?
if(nargin == 6)
    if length(wordlength) == 1
        wordlength=[1 wordlength];
    else
        if length(wordlength) > 2
            error('You can only specify integer and fractional wordlengths')
        end
    end
else
    wordlength = 0;
end

% Check input values
stimulisize=size(inputvalues);
samples=stimulisize(2);
if (stimulisize(1) ~= length(inputids))
    error('Stimuli and ID-mapping inconsistent')
end

inputrows=find(sfg(:,1)==in_op);
if (stimulisize(1) ~= length(inputrows))
    error('Stimuli and signal flow graph inconsistent')
end

[inputs1, ~]=sort(sfg(inputrows,2));
[inputs2, ~]=sort(inputids);
if not(all(inputs1(:) == inputs2(:)))
    error('ID-mapping and signal flow graph inconsistent')
end

outputrows=find(sfg(:,1)==out_op);
outputids=sfg(outputrows,2);
outputcount=length(outputrows);
outputs=zeros(outputcount, samples);
intoutputs=zeros(outputcount, 1);


sfgsize=size(sfg);



% Nodes
nodelist=getnodelist(sfg, s);
nodeids=nodelist;
nodecount=length(nodelist);
intnodes(nodelist)=0;


if nargout > 4
    nodes=zeros(nodecount, samples);
end

% Registers
registerrows=find(sfg(:,1)==delay_op);
registerids=sfg(registerrows,2);
registernodes=sfg(registerrows,3);
registeroutnodes=sfg(registerrows,4);
registercount=length(registerids);
if nargin > 4
    for k = 1:length(initialregisterids)
        intnodes(registeroutnodes(registerids == initialregisterids(k)))=initialregisters(k);
    end
end


if (nargout > 2)
    registers=zeros(registercount, samples);
end




% Simulation loop
for time = 1:samples
    % Operand list
    intinputs=inputvalues(:,time);
    for operand = 1:sfgsize(1)
        opline = sfg(operand,:);
        switch opline(1)
            % Input
            case in_op
                intnodes(opline(3)) = intinputs(inputids==opline(2));
                
                % Output
            case out_op
                intoutputs(outputids==opline(2)) = intnodes(opline(3));
                
                % Addition
            case add_op
                intnodes(opline(5)) = intnodes(opline(3)) + intnodes(opline(4));
                
                % Subtraction
            case sub_op
                intnodes(opline(5)) = intnodes(opline(3)) - intnodes(opline(4));
                
                % Multiplication
            case constmult_op
                intnodes(opline(4)) = intnodes(opline(3)) * opline(5);
                
                % Delay
            case delay_op
                if time > 1
                    intnodes(opline(4)) = intnodes(opline(3));
                end
                % Quantization
            case quant_op
                quant_type = opline(6);
                quant_bits = opline(5);
                switch quant_type
                    case quant_truncation
                        intnodes(opline(4)) = floor((2^(quant_bits-1))*intnodes(opline(3)))/(2^(quant_bits-1));
                    case quant_rounding
                        intnodes(opline(4)) = round((2^(quant_bits-1))*intnodes(opline(3)))/(2^(quant_bits-1));
                    case quant_magnitudetruncation
                        intnodes(opline(4)) = fix((2^(quant_bits-1))*intnodes(opline(3)))/(2^(quant_bits-1));
                    otherwise
                        error('Invalid or non-implemented quantization type')
                end
                
                % Overflow
            case overflow_op
                switch opline(5)
                    case overflow_twosc
                        intnodes(opline(4)) = twoscoverflow(intnodes(opline(3)),1);
                    case overflow_saturation
                        saturation_value = intnodes(opline(3));
                        if saturation_value > 1
                            intnodes(opline(4)) = 1-eps;
                        elseif saturation_value < -1
                            intnodes(opline(4)) = -1;
                        else
                            intnodes(opline(4)) = intnodes(opline(3));
                        end
                    otherwise
                        error('Invalid or non-implemented overflow type')
                end
                
                % Inversion (negation)
            case negate_op
                intnodes(opline(4)) = -intnodes(opline(3));
                
                % Shift
            case shift_op
                shift_amount = opline(5);
                if (opline(6) == shift_left)
                    intnodes(opline(4)) = intnodes(opline(3))*(2^shift_amount);
                else
                    intnodes(opline(4)) = intnodes(opline(3))*(2^(-shift_amount));
                end
                
                % Twoport adaptor
            case twoport_op
                adaptor_type = opline(8);
                switch adaptor_type
                    case adaptor_ser
                        intnodes(opline(5)) = - intnodes(opline(3)) - ...
                            (opline(7))*(intnodes(opline(4)) + ...
                            intnodes(opline(3)));
                        intnodes(opline(6)) = -(intnodes(opline(4)) + ...
                            2*intnodes(opline(3))) ...
                            + (opline(7))*(intnodes(opline(4)) + ...
                            intnodes(opline(3)));
                    case adaptor_par
                        intnodes(opline(5)) = 2*intnodes(opline(4)) - ...
                            intnodes(opline(3)) - (opline(7))* ...
                            (intnodes(opline(4)) - intnodes(opline(3)));
                        intnodes(opline(6)) = intnodes(opline(4)) - ...
                            (opline(7))*(intnodes(opline(4)) - ...
                            intnodes(opline(3)));
                    case adaptor_sym
                        intnodes(opline(5)) = intnodes(opline(4)) + ...
                            (opline(7))*(intnodes(opline(4)) - ...
                            intnodes(opline(3)));
                        intnodes(opline(6)) = intnodes(opline(3)) + ...
                            (opline(7))*(intnodes(opline(4)) - ...
                            intnodes(opline(3)));
                    otherwise
                        error('Invalid or non-implemented adaptor type')
                end
                
                % Division
            case constdivision_op
                intnodes(opline(4)) = intnodes(opline(3)) / ...
                    opline(5);
                
                % Multiplexer
            case mux_op
                if opline(5)
                    intnodes(opline(6)) = intnodes(opline(4));
                else
                    intnodes(opline(6)) = intnodes(opline(3));
                end
                
                % Demultiplexer
            case demux_op
                if opline(4)
                    intnodes(opline(6)) = intnodes(opline(3));
                else
                    intnodes(opline(5)) = intnodes(opline(3));
                end
                
                % Two-operand multiplication
            case mult_op
                intnodes(opline(5)) = intnodes(opline(3)) * intnodes(opline(4));
                
                % Two-operand division
            case division_op
                intnodes(opline(5)) = intnodes(opline(3)) / intnodes(opline(4));
                
                % Reciprocal
            case reciprocal_op
                intnodes(opline(4)) = 1 / intnodes(opline(3));
                
                % Multiply-accumulate
            case constmac_op
                intnodes(opline(5)) = opline(6)*intnodes(opline(3)) + ...
                    intnodes(opline(4));
                
                % Three-operand multiply-accumulate
            case mac_op
                intnodes(opline(6)) = intnodes(opline(3))*intnodes(opline(4)) + ...
                    intnodes(opline(5));
                
                % Butterfly
            case butterfly2_op
                intnodes(opline(4)) = intnodes(opline(2)) + innodes(opline(3));
                intnodes(opline(5)) = intnodes(opline(2)) - innodes(opline(3));
                
                % Constant
            case constant_op
                intnodes(opline(3)) = opline(4);
                
                % Addition/subtraction
            case addsub_op
                addsub_type = opline(6);
                switch addsub_type
                    case addsub_add
                        intnodes(opline(5)) = intnodes(opline(3)) + intnodes(opline(4));
                    case addsub_sub
                        
                        intnodes(opline(5)) = intnodes(opline(3)) - intnodes(opline(4));
                    otherwise
                        error('Invalid or non-implemented add/sub type')                        
                end
                
                % Conjugate
            case conjugate_op
                intnodes(opline(3)) = conj(opline(4));

                % Other operations
            otherwise
                error('Invalid or non-implemented operand')
        end
    end
    
    % Finite wordlength
    if any(wordlength)
        % Truncation
        intnodes = floor(((2^wordlength(2))*intnodes))/(2^wordlength(2));
        intoutputs = floor(((2^wordlength(2))*intoutputs))/(2^wordlength(2));
        % Overflow
        intnodes = twoscoverflow(intnodes, wordlength(1));
        intoutputs = twoscoverflow(intoutputs, wordlength(1));
    end
    
    if registercount && (nargout > 2)
        registers(:,time)=intnodes(registernodes)'; %'
    end
    
    outputs(:,time)=intoutputs;
    if nargout > 4
        nodes(:,time)=intnodes(nodelist)'; %'
    end
    
end



